
;---------------------------------------------------------------------------------------------------------------
; Unsigned 24-bit conversion, programmable decimal point. 390..740 cycles depending on input binary word value
u24toap:
        seh             ; Force decimal point placement
u24toa_e:                       ; Unsigned 24-bit conversion routine internal entry point
        clc             ; Force 24-bit conversion
uitoa_e:                        ; Unsigned 16-bit conversion routine internal entry point
        clt             ; Force unsigned conversion
sitoa_e:                        ; Signed 16-bit conversion routine internal entry point
        push    r2      ; Save registers
        push    r3      ;
        push    r4      ;
        push    r5      ;
        push    r6      ;
        push    r7      ;
        push    wreg    ;
        push    wreg1   ;
        push    wreg2   ;
        push    yl      ;
        push    yh      ;
        push    zl      ;
        push    zh      ;

        brhs    xtoa1           ; Check if decimal point is required
        ldi     wreg,0          ; Clear wreg if not (DO NOT USE clr wreg - it trashes CY flag!)
xtoa1:
        brcc    u24conv         ; Check if 24-bit conversion routine was called
        clr     r4              ; If not, clear a tmpbin MSB
        ldi     wreg1,5         ; and set ASCII byte counter to 5
        rjmp    x16conv

u24conv:
        ldi     wreg1,8         ; Set ASCII byte counter to 8
        ld      r4,y+           ; Read a binary MSB to r4 (= tmpbin MSB)
x16conv:
        ld      r3,y+           ; Read a binary 2SB to r3 (= tmpbin 2SB for 24-bit, = MSB for 16-bit)
        ld      r2,y            ; Read a binary LSB to r2 (= tmpbin LSB)

        brtc    i16_pos         ; Process a binary number as unsigned if T bit was not set before
        sbrs    r3,7            ; Process a negative 16-bit binary number if its MSb = 1,
        rjmp    i16_pos         ; otherwise process it as unsigned
        ldi     wreg2,'-'       ; Place a '-' character at the beginning of ASCII string
        st      z+,wreg2        ;
        clr     wreg2           ; Negate a negative 16-bit tmpbin value to make it positive
        neg     r2              ;
        sbc     wreg2,r3        ;
        mov     r3,wreg2        ;

i16_pos:
	clt		        ; Enable leading zero blanking
        movw    yl,zl           ; Copy Z ptr to X ptr
        ldiw    z,(n10tab*2)    ; Z points to a 10^N 8*3 byte table


        sbrs    wreg1,3         ; Check if a 16-bit conversion routine was called
        adiw    zl,3*3          ; If yes, add 3*3 to Z so it points to 10,000 constant instead of 10,000,000 one

i24_loop:
        mov     wreg2,wreg1     ; Check if a DP should be set on next ASCII position
        dec     wreg2           ;
        cp      wreg2,wreg      ; Compare a DP position register with next ASCII digit position
        brne    skipzp          ; Skip if not equal
        set                     ; Disable zero blanking from now on to make 0.xxx reading

skipzp:
        cp      wreg1,wreg      ; Compare a DP position register with current ASCII digit position
        brne    skipdp          ; Skip if not equal
        ldi     wreg2,'.'       ; Load a '.' character to wreg2
        st      y+,wreg2        ; Store a '.' to a current ASCII digit position

skipdp:
        lpm     r7,z+   ; r7 = 10^N MSB
        lpm     r6,z+   ; r6 = 10^N 2SB
        lpm     r5,z+   ; r5 = 10^N LSB
	ldi	wreg2,'0'-1     ; wreg2 = ASCII Nth digit = '0'-1

sub10n:
        inc     wreg2   ; ASCII Nth digit = '0','1','2','3','4','5','6','7','8','9'
	sub	r2,r5   ; tmpbin = tmpbin - 10^N
	sbc	r3,r6   ;
	sbc	r4,r7   ;
	brsh	sub10n  ; Keep subtracting 10^N from tmpbin and incrementing ASCII digit while tmpbin >= 0

	add	r2,r5   ; Restore tmpbin (tmpbin = tmpbin + 10^N)
	adc	r3,r6   ;
	adc	r4,r7   ;
        st      y+,wreg2; Store a next ASCII digit

        cp      wreg1,wreg      ; Compare a current ASCII digit position with DP position register
        brsh    skipzbd         ; Skip if not equal
        set                     ; Disable further zero blanking from an ASCII position before the DP position

skipzbd:
        brts    i24chk  ; Skip if zero blanking already disabled
	set		; Preset a zero blanking disable flag
        cpi     wreg2,'0'       ; Check if a current ASCII digit is '0'
	brne	i24chk	; If not, continue converting with zero blanking disabled
	clt		; If yes, allow zero blanking again
        sbiw    yl,1    ; and decrement ASCII pointer to overwrite the last stored '0' with next ASCII digit

i24chk:
        dec     wreg1           ; Decrement wreg1 (ASCII digit counter)
        brne    i24_loop        ; Repeat until wreg1 is not zero

        brts    stzterm ; Keep at least one '0' if tmpbin was = 0
        adiw    yl,1    ;

stzterm:
        st      y,wreg1 ; Put a binary zero terminator after a last ASCII digit

        pop     zh      ; Restore registers
        pop     zl      ;
        pop     yh      ;
        pop     yl      ;
        pop     wreg2   ;
        pop     wreg1   ;
        pop     wreg    ;
        pop     r7      ;
        pop     r6      ;
        pop     r5      ;
        pop     r4      ;
        pop     r3      ;
        pop     r2      ;

        ret

.include	"0n10tab.asm"
