;***************************************************************************************************************
; 16- and 24-bit binary to ASCIIZ with leading zero suppression conversion routines:
;
; uitoa   - unsigned 16-bit conversion, no decimal point is put into ASCIIZ string
; uitoap  - unsigned 16-bit conversion, programmable decimal point is put into ASCIIZ string
; sitoa   - signed 16-bit conversion, no decimal point is put into ASCIIZ string
; sitoap  - signed 16-bit conversion, programmable decimal point is put into ASCIIZ string
; u24toa  - unsigned 24-bit conversion, no decimal point is put into ASCIIZ string
; u24toap - unsigned 24-bit conversion, programmable decimal point is put into ASCIIZ string
;
; Input: Y    = RAM address pointer to MSB of a binary value to be converted (little endian),
;        Z    = RAM address pointer to a start (MSD) of an output ASCIIZ string (little endian),
;        wreg = # of digits after a decimal point (DP) for XXXtoaP routines (0 = no DP)
;
; Output: ASCIIZ string (little endian) in a RAM starting with an MSD pointed by Z
;         with leading zeroes suppressed (and with wreg ASCII characters after DP for XXXtoaP routines).
;
; If no DP was selected, an ASCIIZ string "0" will be returned for zero binary value.
; If a DP was selected (e.g. at 3rd position), an ASCIIZ string "0.000" will be returned for zero binary value.
;
; All registers are saved/restored.
;
; 119 program words, 280..740 cycles depending on input binary word size/value.
;---------------------------------------------------------------------------------------------------------------

;---------------------------------------------------------------------------------------------------------------
; Unsigned 16-bit conversion, no decimal point. 280..500 cycles depending on input binary word value
uitoa:
        clh                     ; Force no decimal point placement
        sec                     ; Force 16-bit conversion
        rjmp    uitoa_e         ; Jump to unsigned 16-bit conversion routine entry
;---------------------------------------------------------------------------------------------------------------

;---------------------------------------------------------------------------------------------------------------
; Unsigned 16-bit conversion, programmable decimal point. 280..500 cycles depending on input binary word value
uitoap:
        seh                     ; Force decimal point placement
        sec                     ; Force 16-bit conversion
        rjmp    uitoa_e         ; Jump to unsigned 16-bit conversion routine entry
;---------------------------------------------------------------------------------------------------------------

;---------------------------------------------------------------------------------------------------------------
; Signed 16-bit conversion, no decimal point. 280..500 cycles depending on input binary word value
sitoa:
        clh                     ; Force no decimal point placement
        sec                     ; Force 16-bit conversion
        set                     ; Force signed conversion
        rjmp    sitoa_e         ; Jump to signed 16-bit conversion routine entry
;---------------------------------------------------------------------------------------------------------------

;---------------------------------------------------------------------------------------------------------------
; Signed 16-bit conversion, programmable decimal point. 280..500 cycles depending on input binary word value
sitoap:
        seh                     ; Force decimal point placement
        sec                     ; Force 16-bit conversion
        set                     ; Force signed conversion
        rjmp    sitoa_e         ; Jump to signed 16-bit conversion routine entry
;---------------------------------------------------------------------------------------------------------------

;---------------------------------------------------------------------------------------------------------------
; Unsigned 24-bit conversion, no decimal point. 390..740 cycles depending on input binary word value
u24toa:
        clh                     ; Force no decimal point placement
        rjmp    u24toa_e        ; Jump to unsigned 24-bit conversion routine entry
;---------------------------------------------------------------------------------------------------------------

