.nolist
.include        "m48def.inc"
.list

;---------------------------------------------------------------------------------------------------------------
.macro	ldiw            ; Macro for loading a 16-bit immediate value to xh:xl/yh:yl/zh:zl register pair
	ldi	@0l,low(@1)
	ldi	@0h,high(@1)
.endm


.def	wreg	= r16   ; Define working registers
.def	wreg1	= r17   ; 
.def	wreg2	= r18   ;


.dseg
binbuf: .byte   3       ; Binary number location in RAM, 3 bytes (MSB first)
ascbuf: .byte   10      ; ASCIIZ string location in RAM, 10 bytes (MSD first)

.cseg

;---------------------------------------------------------------------------------------------------------------
; Test code for binary to ASCIIZ conversion routines
xtoa_test:

.equ    unsigned24 = 15999999   ; unsigned 24-bit test number, 0..16777215 range
.equ    unsigned16 = 64999      ; unsigned 16-bit test number, 0..65535 range
.equ    signed16 = 31999        ; signed 16-bit test number, 0..32767 range

                ldiw    y,binbuf                ; Y points to an input binary MSB
                ldiw    z,ascbuf                ; Z points to an output ASCIIZ string start

		ldi	wreg,byte3(unsigned24)	; 24-bit binary MSB
                std     y+0,wreg
		ldi	wreg,byte2(unsigned24)	; 24-bit binary 2SB
                std     y+1,wreg
		ldi	wreg,byte1(unsigned24)	; 24-bit binary LSB
                std     y+2,wreg
		ldi	wreg,4                  ; Decimal point (DP) position
		rcall	u24toap 		; Test an unsigned 24-bit conversion

		ldi	wreg,high(unsigned16)	; 16-bit binary MSB
                std     y+0,wreg
		ldi	wreg,low(unsigned16)	; 16-bit binary LSB
                std     y+1,wreg
		ldi	wreg,3                  ; Decimal point (DP) position
		rcall	uitoap	        	; Test an unsigned 16-bit conversion

		ldi	wreg,high(signed16)	; 16-bit binary MSB
                std     y+0,wreg
		ldi	wreg,low(signed16)	; 16-bit binary LSB
                std     y+1,wreg
		ldi	wreg,2                  ; Decimal point (DP) position
		rcall	sitoap	        	; Test a signed 16-bit conversion with positive value

		ldi	wreg,high(-signed16)	; 16-bit binary MSB
                std     y+0,wreg
		ldi	wreg,low(-signed16)	; 16-bit binary LSB
                std     y+1,wreg
		ldi	wreg,1                  ; Decimal point (DP) position
		rcall	sitoap	        	; Test a signed 16-bit conversion with negative value

		rjmp    xtoa_test               ; Loop
;---------------------------------------------------------------------------------------------------------------

;***************************************************************************************************************
; 16- and 24-bit binary to ASCIIZ with leading zero suppression conversion routines:
;
; uitoa   - unsigned 16-bit conversion, no decimal point is put into ASCIIZ string
; uitoap  - unsigned 16-bit conversion, programmable decimal point is put into ASCIIZ string
; sitoa   - signed 16-bit conversion, no decimal point is put into ASCIIZ string
; sitoap  - signed 16-bit conversion, programmable decimal point is put into ASCIIZ string
; u24toa  - unsigned 24-bit conversion, no decimal point is put into ASCIIZ string
; u24toap - unsigned 24-bit conversion, programmable decimal point is put into ASCIIZ string
;
; Input: Y    = RAM address pointer to MSB of a binary value to be converted (little endian),
;        Z    = RAM address pointer to a start (MSD) of an output ASCIIZ string (little endian),
;        wreg = # of digits after a decimal point (DP) for XXXtoaP routines (0 = no DP)
;
; Output: ASCIIZ string (little endian) in a RAM starting with an MSD pointed by Z
;         with leading zeroes suppressed (and with wreg ASCII characters after DP for XXXtoaP routines).
;
; If no DP was selected, an ASCIIZ string "0" will be returned for zero binary value.
; If a DP was selected (e.g. at 3rd position), an ASCIIZ string "0.000" will be returned for zero binary value.
;
; All registers are saved/restored.
;
; 119 program words, 280..740 cycles depending on input binary word size/value.
;---------------------------------------------------------------------------------------------------------------

;---------------------------------------------------------------------------------------------------------------
; Unsigned 16-bit conversion, no decimal point. 280..500 cycles depending on input binary word value
uitoa:
        clh                     ; Force no decimal point placement
        sec                     ; Force 16-bit conversion
        rjmp    uitoa_e         ; Jump to unsigned 16-bit conversion routine entry
;---------------------------------------------------------------------------------------------------------------

;---------------------------------------------------------------------------------------------------------------
; Unsigned 16-bit conversion, programmable decimal point. 280..500 cycles depending on input binary word value
uitoap:
        seh                     ; Force decimal point placement
        sec                     ; Force 16-bit conversion
        rjmp    uitoa_e         ; Jump to unsigned 16-bit conversion routine entry
;---------------------------------------------------------------------------------------------------------------

;---------------------------------------------------------------------------------------------------------------
; Signed 16-bit conversion, no decimal point. 280..500 cycles depending on input binary word value
sitoa:
        clh                     ; Force no decimal point placement
        sec                     ; Force 16-bit conversion
        set                     ; Force signed conversion
        rjmp    sitoa_e         ; Jump to signed 16-bit conversion routine entry
;---------------------------------------------------------------------------------------------------------------

;---------------------------------------------------------------------------------------------------------------
; Signed 16-bit conversion, programmable decimal point. 280..500 cycles depending on input binary word value
sitoap:
        seh                     ; Force decimal point placement
        sec                     ; Force 16-bit conversion
        set                     ; Force signed conversion
        rjmp    sitoa_e         ; Jump to signed 16-bit conversion routine entry
;---------------------------------------------------------------------------------------------------------------

;---------------------------------------------------------------------------------------------------------------
; Unsigned 24-bit conversion, no decimal point. 390..740 cycles depending on input binary word value
u24toa:
        clh                     ; Force no decimal point placement
        rjmp    u24toa_e        ; Jump to unsigned 24-bit conversion routine entry
;---------------------------------------------------------------------------------------------------------------

;---------------------------------------------------------------------------------------------------------------
; Unsigned 24-bit conversion, programmable decimal point. 390..740 cycles depending on input binary word value
u24toap:
        seh             ; Force decimal point placement
u24toa_e:                       ; Unsigned 24-bit conversion routine internal entry point
        clc             ; Force 24-bit conversion
uitoa_e:                        ; Unsigned 16-bit conversion routine internal entry point
        clt             ; Force unsigned conversion
sitoa_e:                        ; Signed 16-bit conversion routine internal entry point
        push    r2      ; Save registers
        push    r3      ;
        push    r4      ;
        push    r5      ;
        push    r6      ;
        push    r7      ;
        push    wreg    ;
        push    wreg1   ;
        push    wreg2   ;
        push    yl      ;
        push    yh      ;
        push    zl      ;
        push    zh      ;

        brhs    xtoa1           ; Check if decimal point is required
        ldi     wreg,0          ; Clear wreg if not (DO NOT USE clr wreg - it trashes CY flag!)
xtoa1:
        brcc    u24conv         ; Check if 24-bit conversion routine was called
        clr     r4              ; If not, clear a tmpbin MSB
        ldi     wreg1,5         ; and set ASCII byte counter to 5
        rjmp    x16conv

u24conv:
        ldi     wreg1,8         ; Set ASCII byte counter to 8
        ld      r4,y+           ; Read a binary MSB to r4 (= tmpbin MSB)
x16conv:
        ld      r3,y+           ; Read a binary 2SB to r3 (= tmpbin 2SB for 24-bit, = MSB for 16-bit)
        ld      r2,y            ; Read a binary LSB to r2 (= tmpbin LSB)

        brtc    i16_pos         ; Process a binary number as unsigned if T bit was not set before
        sbrs    r3,7            ; Process a negative 16-bit binary number if its MSb = 1,
        rjmp    i16_pos         ; otherwise process it as unsigned
        ldi     wreg2,'-'       ; Place a '-' character at the beginning of ASCII string
        st      z+,wreg2        ;
        clr     wreg2           ; Negate a negative 16-bit tmpbin value to make it positive
        neg     r2              ;
        sbc     wreg2,r3        ;
        mov     r3,wreg2        ;

i16_pos:
	clt		        ; Enable leading zero blanking
        movw    yl,zl           ; Copy Z ptr to X ptr
        ldiw    z,(n10tab*2)    ; Z points to a 10^N 8*3 byte table


        sbrs    wreg1,3         ; Check if a 16-bit conversion routine was called
        adiw    zl,3*3          ; If yes, add 3*3 to Z so it points to 10,000 constant instead of 10,000,000 one

i24_loop:
        mov     wreg2,wreg1     ; Check if a DP should be set on next ASCII position
        dec     wreg2           ;
        cp      wreg2,wreg      ; Compare a DP position register with next ASCII digit position
        brne    skipzp          ; Skip if not equal
        set                     ; Disable zero blanking from now on to make 0.xxx reading

skipzp:
        cp      wreg1,wreg      ; Compare a DP position register with current ASCII digit position
        brne    skipdp          ; Skip if not equal
        ldi     wreg2,'.'       ; Load a '.' character to wreg2
        st      y+,wreg2        ; Store a '.' to a current ASCII digit position

skipdp:
        lpm     r7,z+   ; r7 = 10^N MSB
        lpm     r6,z+   ; r6 = 10^N 2SB
        lpm     r5,z+   ; r5 = 10^N LSB
	ldi	wreg2,'0'-1     ; wreg2 = ASCII Nth digit = '0'-1

sub10n:
        inc     wreg2   ; ASCII Nth digit = '0','1','2','3','4','5','6','7','8','9'
	sub	r2,r5   ; tmpbin = tmpbin - 10^N
	sbc	r3,r6   ;
	sbc	r4,r7   ;
	brsh	sub10n  ; Keep subtracting 10^N from tmpbin and incrementing ASCII digit while tmpbin >= 0

	add	r2,r5   ; Restore tmpbin (tmpbin = tmpbin + 10^N)
	adc	r3,r6   ;
	adc	r4,r7   ;
        st      y+,wreg2; Store a next ASCII digit

        cp      wreg1,wreg      ; Compare a current ASCII digit position with DP position register
        brsh    skipzbd         ; Skip if not equal
        set                     ; Disable further zero blanking from an ASCII position before the DP position

skipzbd:
        brts    i24chk  ; Skip if zero blanking already disabled
	set		; Preset a zero blanking disable flag
        cpi     wreg2,'0'       ; Check if a current ASCII digit is '0'
	brne	i24chk	; If not, continue converting with zero blanking disabled
	clt		; If yes, allow zero blanking again
        sbiw    yl,1    ; and decrement ASCII pointer to overwrite the last stored '0' with next ASCII digit

i24chk:
        dec     wreg1           ; Decrement wreg1 (ASCII digit counter)
        brne    i24_loop        ; Repeat until wreg1 is not zero

        brts    stzterm ; Keep at least one '0' if tmpbin was = 0
        adiw    yl,1    ;

stzterm:
        st      y,wreg1 ; Put a binary zero terminator after a last ASCII digit

        pop     zh      ; Restore registers
        pop     zl      ;
        pop     yh      ;
        pop     yl      ;
        pop     wreg2   ;
        pop     wreg1   ;
        pop     wreg    ;
        pop     r7      ;
        pop     r6      ;
        pop     r5      ;
        pop     r4      ;
        pop     r3      ;
        pop     r2      ;

        ret

; 10^N constants table. DO NOT USE SEPARATE .db because table elements are packed 3 bytes long!
n10tab: .db     byte3(10000000),byte2(10000000),byte1(10000000),\
                byte3(1000000),byte2(1000000),byte1(1000000),\
                byte3(100000),byte2(100000),byte1(100000),\
                byte3(10000),byte2(10000),byte1(10000),\
                byte3(1000),byte2(1000),byte1(1000),\
                byte3(100),byte2(100),byte1(100),\
                byte3(10),byte2(10),byte1(10),\
                byte3(1),byte2(1),byte1(1)
;***************************************************************************************************************


